/*
	Ext extensions
	
	Copyright (c) 2008 Aparajita Fishman, All Rights Reserved (except Ext.ux.SearchField)
	
	This code provides some wrappers around common Ext JS functionality, and some nice
	utility routines you are free to use in your code. This code is provided
	on an "as is" basis with no warranty as to its behavior.
*/

// Create a default App config
App = {
	name: 'My App'
};


Ext.namespace('Ext.ux');

Ext.ux.Data = function() {

	var timeoutTpl = new Ext.Template(
		'<div class="ux-msgbox">',
		'<p>You tried to access {name}, but you ended up here because:</p>',
		'<ul>',
			'<li>You do not have cookies turned on in your browser.</li>',
			'<li>You had logged in, but did not access our server for at least {timeout} minutes.</li>',
		'</ul>',
		'<p>To login again, click the Login button below.</p>',
		'</div>');
	timeoutTpl = timeoutTpl.compile();
	timedOut = false;
	
	
	function _updateRecord(button, text, options) {
		if (button === 'yes' || button === 'ok') {
			Ext.ux.Msg.wait(options.title || '', options.progressMsg || 'Please wait...');
		
			var opts = Ext.apply({}, options);
			Ext.apply(opts, options.request);
			Ext.apply(opts, {
				mode: 'submit',
				success: null,
				failure: null,
				callback: Ext.ux.Data.checkActionResult
			});
		
			Ext.Ajax.request(opts);
		}
	}

	
	
	return {
		
		sessionTimeout: 10,
		
			
		/*
			Convenience function for updating records. Takes the following options:
		
			confirmOptions : Object		If present, is passed to Ext.Msg.show
			recordName : String			A textual name for the object being updated
			title : String				Title for the progress message dialog
			progressMsg : String		Used as the progress message while the request is pending
			request : Object			Passed to Ajax.request. The callbacks should be defined here.
										The callbacks will be called with the parameters 
										(success : Boolean, json : Object, userData : Object)
			scope : Object				The scope used by all callbacks, including the request
			successMsg : String			If present, an alert with this message is displayed on success
			failureMsg : String			If present, an alert with this message is displayed on failure
			userData : Mixed			Extra data that can be passed to the request callbacks
		*/
		updateRecord: function(options) {
		
			if (options.confirmOptions) {
				var confirmOptions = options.confirmOptions;
				confirmOptions.fn = _updateRecord.createDelegate(options.scope || this, [options], true);
				Ext.Msg.show(confirmOptions);
			}
			else {
				_updateRecord('ok', '', options);
			}
		},
	
	
		checkActionResult: function(options, success, response) {
		
			if (options.keepProgress !== true) {
				Ext.ux.Msg.hide();	// Hide the progress dialog if it's still there
			}
		
			var errMsg = null;
		
			if (success) {
				var json = null;
			
				// See if we have a JSON response. If so,
				// the first non-whitespace character should be '{' or '['.
				if (/^\s*[{[]/.test(response.responseText)) {
					try {
						json = Ext.util.JSON.decode(response.responseText);
					}
					catch (e) {
						json = null;
					}
				
					// If there is a closer, do it now
					Ext.ux.Util.close(options.closer);
				
					if (Ext.type(json.result) === 'string') {
						var callSuccess = function(json, options) {
							if (Ext.type(options.request.success) === 'function') {
								options.request.success.call(this, true, json, options.userData);
							}
					
							if (Ext.type(options.request.callback) === 'function') {
								options.request.callback.call(this, true, json, options.userData);
							}
						}.createDelegate(options.scope || this, [json, options]);
				
						var callFailure = function(json, options) {
							if (Ext.type(options.request.failure) === 'function') {
								options.request.failure.call(this, false, json, options.userData);
							}
					
							if (Ext.type(options.request.callback) === 'function') {
								options.request.callback.call(this, true, json, options.userData);
							}
						}.createDelegate(options.scope || this, [json, options]);
				
						var name = options.recordName ? 'The ' + options.recordName : 'That record';
				
						switch (json.result) {
							case 'success':
								if (options.successMsg) {
									Ext.ux.Msg.alertInfo(
										'', options.successMsg,
										callSuccess, 
										options.scope);
								}
								else {
									callSuccess();
								}
								break;
					
							case 'locked':
								errMsg = name + ' is currently being edited by someone else. Please try again later.';
								success = false;
								break;
						
							case 'conflict':
								errMsg = name + ' was modified by someone else while you were editing. Please try again later.';
								success = false;
								break;
					
							case 'timeout':
								Ext.ux.Data.showTimeout();
								return false;
						
							case 'missing':
								errMsg = name + ' cannot be found in the database. It may have been deleted.';
								success = false;
								break;
						
							default:
								success = false;
						
								if (json.message) {
									errMsg = json.message;
								}
								break;
						}
					}
					else {
						success = false;
					}
				}
				else {
					// Response is not JSON. Unless the mode is 'direct' an error occurred.
					if (options.mode != 'direct') {
						success = false;
					}
				}
			}
		
			if (!success) {
				Ext.ux.Msg.alertError(
					'Error', 
					errMsg || options.failureMsg || 'An error occurred. Please try again later.',
					callFailure,
					options.scope || this);
			}
		
			return success;
		},
	
	
		showTimeout: function(closer) {
			Ext.ux.Data.timedOut = true;
			Ext.ux.Util.close(closer);
		
			Ext.Msg.show({
				title: 'Session Timeout',
				width: 500,
				closable: false,
				cls: 'static',
				icon: Ext.Msg.INFO,
				modal: true,
				msg: timeoutTpl.apply({ name: App.name, timeout: Ext.ux.Data.sessionTimeout }),
			
				buttons: { ok: 'Login', cancel: 'No thanks' },
			
				fn: function(button) {
					if (button === 'ok') {
						App.loginFunction();
					}
					else {
						App.logoutFunction();
					}
				}
			});
		}
	};
}();


/****************************************************
	ux.data.JsonReader
	
	Subclass of JsonReader that looks for a 'timeout'
	property in the response JSON. If it exists and
	is true, then the timeout handler is invoked.
****************************************************/
Ext.namespace('Ext.ux.data');

Ext.ux.data.JsonReader = function(mode, closer, meta, recordType) {
	this.uxMode = mode;
	this.uxCloser = closer;
	meta = meta || {};
	recordType = recordType || {};
	Ext.ux.data.JsonReader.superclass.constructor.call(this, meta, recordType);
};

Ext.extend(Ext.ux.data.JsonReader, Ext.data.JsonReader, {
	readRecords : function(o){
		var timeout = o.result === 'timeout';
		var result = {
			success: !timeout
		};
		
		switch (this.uxMode) {
			case 'select':
				if (!timeout) {
					return Ext.ux.data.JsonReader.superclass.readRecords.apply(this, arguments);
				}
				else {
					Ext.ux.Data.showTimeout(this.uxCloser);
					throw 'timeout';
				}
				
			case 'load':
				/*
					When using a custom reader for a load action, the result is expected
					to be in the format:
					
					success: true,
					records: [{
						field1: 'value1',
						field2: 'value2'
					}]
					
					But an object called 'data' is the documented interface for field values 
					when no reader is specified. Clients on the back end shouldn't know or care
					if a custom reader is being used, so we transform the 'data' field into 'records'.
				*/
				if (timeout) {
					Ext.ux.Data.showTimeout(this.uxCloser);
				}
				
				result.records = o.result == 'success' ? [{data: o.data}] : {};
				return result;
				
			case 'submit':				
				/*
					When using a custom reader for a submit action, an error result is expected
					in the format:
					
					success: false,
					records: [{
						data: {
							id: 'field1_id',
							msg: 'Field1 is not valid.'
						}
					},
					{
						data: {
							id: 'field2_id',
							msg: 'Field2 is not valid either.'
						}
					}]
					
					The documented interface is:
					
					success: false,
					errors: {
						field1_id: 'Field1 is not valid.',
						field2_id: 'Field2 is not valid either.'
					}
					
					We want to hide this from clients, so we transform the 'errors' field
					into the 'records' array. If the client returned a result code
					other than success, the first record is:
					
					{
						__<result code>__: ''
					}
				*/
				
				if (o.result == 'success') {
					result.records = [];
					
					for (var field in o.data) {
						if (o.data.hasOwnProperty(field)) {
							result.records.push({
								data: { 
									id: "'" + field + "'", 
									msg: o.data[field] 
								}
							});
						}
					}
				}
				else {
					result.records = [{
						data: {
							id: String.format("'__{0}__'", o.result),
							msg: ''
						}
					}];
				}
				
				return result;
				
			default:
				throw 'timeout';	// bail
		}
	}
});


/****************************************************
	ux.data.JsonStore
	
	Subclass of JsonStore that looks for timeouts
	and will optionally try to close a window.
****************************************************/
Ext.ux.data.JsonStore = function(c, close){
	 Ext.ux.data.JsonStore.superclass.constructor.call(this, Ext.apply(c, {
		  proxy: !c.data ? new Ext.data.HttpProxy({url: c.url}) : undefined,
		  reader: new Ext.ux.data.JsonReader('select', close, c, c.fields)
	 }));
};
Ext.extend(Ext.ux.data.JsonStore, Ext.data.Store);


Ext.ux.Msg = {	
	
	waitId: 0,
	
	
	alertInfo: function(title, prompt, callback, scope, animEl) {
		Ext.ux.Msg.alert(title, prompt, Ext.Msg.INFO, callback, scope, animEl);
	},
	
	
	alertWarning: function(title, prompt, callback, scope, animEl) {
		Ext.ux.Msg.alert(title, prompt, Ext.Msg.WARNING, callback, scope, animEl);
	},
	
	
	alertError: function(title, prompt, callback, scope, animEl) {
		Ext.ux.Msg.alert(title, prompt, Ext.Msg.ERROR, callback, scope, animEl);
	},
	
	
	alert: function(title, prompt, icon, callback, scope, animEl) {
		Ext.Msg.show({
			title: title,
			msg: prompt,
			buttons: Ext.Msg.OK,
			icon: icon,
			animEl: animEl,
			fn: callback ? callback.createDelegate(scope || this) : null
		});
	},
	
	
	wait: function(title, msg, progressText, interval, animEl) {
		Ext.Msg.show({
			title: title || App.name,
			msg: msg,
			progressText: progressText || '',
			width: 400,
			wait: true,
			waitConfig: { interval: interval ? interval : 200 },
			animEl: animEl
		});
		
		// Add a unique identifier to this invocation of wait().
		// This is useful in hide().
		++Ext.ux.Msg.waitId;
		Ext.Msg.getDialog().waitId = Ext.ux.Msg.waitId;
	},
	
	
	/*
		When hiding a progress dialog, it's nice to complete the progress to
		100%, wait a half second, then close it completely.
	*/
	hide: function(waitId) {
		if (!Ext.Msg.isVisible()) {
			return;
		}
		
		// First make sure it actually is a progress dialog
		var dlg = Ext.Msg.getDialog();
		var progress = dlg.getEl().child('.x-progress-wrap');
		
		if (!progress || !progress.isVisible()) {
			// If we were passed a waitId, that means we wanted to close a progress dialog
			// which has been replaced by an alert during the 500ms delay. In that case
			// don't close the alert!
			if (Ext.type(waitId) != 'number') {
				Ext.Msg.hide();
			}
		}
		
		// If we were passed a waitId and it matches the current progress dialog, close it.
		else if (Ext.type(waitId) == 'number' && waitId === dlg.waitId) {
			Ext.Msg.hide();
		}
		else {
			progress = Ext.getCmp(progress.id);
		
			if (progress.isWaiting()) {
				// This is a hack, I'm relying on internal structures
				progress.waitTimer.onStop = null; 
				Ext.TaskMgr.stop(progress.waitTimer);
				progress.waitTimer = null;
			}
		
			Ext.Msg.updateProgress(1);
		
			// Now wait half a second, and then hide the dialog
			Ext.ux.Msg.hide.defer(500, Ext.ux.Msg, [dlg.waitId]);
		}
	}
	
};

	
Ext.ux.Util = {
	
	getButtonWindow: function(button) {
		var id = button.getEl().findParent('div.x-window', 2700).id;
		return Ext.getCmp(id);
	},
	
	
	getSelectedOptions: function(select) {
		var selected = [];
		var options = select.options;
		
		for (var i = 0; i < options.length; ++i) {
			if (options[i].selected) {
				selected.push(i);
			}
		}
		
		return selected;
	},
	
	
	addArguments: function(args) {
		var a = Array.prototype.slice.call(args, 0);
		
		for (var i = 1; i < arguments.length; ++i) {
			a.push(arguments[i]);
		}
		
		return a;
	},
	
	
	close: function(closer) {
		var type = Ext.type(closer);
		
		switch (type) {
			case 'string':
				var w = Ext.getCmp(closer);
				
				if (w) {
					w.close();
				}
				break;
				
			case 'function':
				closer();
				break;
				
			case 'object':
				closer.close();
				break;
				
			default:
				break;
		}
	}
};


Ext.ux.PDFPluginChecker = function() {
	
	// NS3+, Opera3+, IE5+ Mac, Safari (support plugin array):	
	// check for Acrobat plugin in plugin array
	if (navigator.plugins !== null && navigator.plugins.length > 0) {
		var mimeType = navigator.mimeTypes["application/pdf"];

		if (mimeType) {
			// PDF is supported, is there a plugin for PDF?
			if (mimeType.enabledPlugin) {
				// On Safari, we need to handle the doc viewer window differently
				if (Ext.isSafari) {
					this.pdfVersion = this.pdfVersion_Safari;
				}
				else {
					this.pdfVersion = this.pdfVersion_Normal;
				}
			}
		}

	}
	else if (Ext.isIE) {
		// As usual Microsoft does not support standards and requires all sorts
		// of nonsense to figure out the PDF version.
		
		var oDoc = null;

		try
		{
			oDoc = new ActiveXObject('AcroExch.Document');	// version 7+ check

			if (oDoc)
				this.pdfVersion = 7;
		}
		catch (e)
		{
			// not installed, try version 5/6
		}

		if (this.pdfVersion == this.pdfVersion_DontKnow)
		{
			try
			{
				oDoc = new ActiveXObject('PDF.PdfCtrl.5');	// version 5/6 check

				if (oDoc)
					this.pdfVersion = 5;
			}
			catch (e)
			{
				// not installed, try version 4
			}
		}

		if (this.pdfVersion == this.pdfVersion_DontKnow)
		{
			try
			{
				oDoc = new ActiveXObject('PDF.PdfCtrl.1');	// version 4 check

				if (oDoc)
					this.pdfVersion = 4;
			}
			catch (e)
			{
				// no plugin
			}
		}

		// window.focus() is broken on IE 6 currently, we have to special case it
		if (this.pdfVersion != this.pdfVersion_DontKnow && Ext.isIE6)
			this.pdfVersion = this.pdfVersion_IE6;
	}
};

Ext.ux.PDFPluginChecker.prototype = {
	
	pdfVersion_DontKnow: -1,
	pdfVersion_Normal: 1,
	pdfVersion_Safari: 1010,
	pdfVersion_IE6: 666,
	pdfVersion: Ext.ux.PDFPluginChecker.pdfVersion_DontKnow,
	updateOptions: null,
	
	
	getPDFVersion: function() {
		return this.pdfVersion;
	}
};

Ext.namespace('Ext.ux.form');

Ext.ux.form.SubmitHandler = function() {};
Ext.ux.form.SubmitHandler.prototype = {
	
	originalOptions: {},
	
	
	submit: function(options) {
		// Save the original options so we can get the values later
		this.originalOptions = Ext.apply({}, options);
		
		var opts = Ext.apply(options, {
			success: this.onSubmitSuccess,
			failure: this.onSubmitFailure,
			scope: this
		});
		
		opts.form.submit(opts);
	},
	
	
	onSubmitSuccess: function(form, action) {
		var cleanup = function() {
			Ext.ux.Util.close(this.originalOptions.closer);
			
			if (this.originalOptions.success) {
				this.originalOptions.success.apply(this.originalOptions.scope, arguments);
			}
		}.createDelegate(this);
		
		if (this.originalOptions.successMsg) {
			Ext.ux.Msg.alertInfo('', this.originalOptions.successMsg, cleanup);
		}
		else {
			cleanup();
		}
		
	},
	
	
	onSubmitFailure: function(form, action) {
		var cleanup = function() {
			Ext.ux.Util.close(this.originalOptions.closer);
			
			if (this.originalOptions.failure) {
				this.originalOptions.failure.apply(this.originalOptions.scope, arguments);
			}
		}.createDelegate(this);
		
		var error = this.originalOptions.errorMsg || 'An error occurred. Please try again later.';
		
		if (action.result) {
			var opts = {
				request: {},
				scope: this,
				recordName: '',
				closer: cleanup,
				mode: 'submit'
			};

			if (!Ext.ux.Data.checkActionResult(opts, true, action.response)) {
				// Timeout dialog is now showing, don't show error alert
				return;
			}
			
			if (action.result.message) {
				error = action.result.message || error;
			}
		}
		
		if (error) {
			Ext.ux.Msg.alertError('Error', error, cleanup);
		}
		else {
			cleanup();
		}
	}
};


Ext.namespace('Ext.ux.util');

Ext.ux.util.Format = function() {
	
	function formatReplace(fullMatch, selector, choice1, choice2, testValue, args) {
		if (!fullMatch) {
			return '';
		}
		
		var useFirst;
		selector = args[parseInt(selector, 10)];
		var type = Ext.type(selector);
		
		switch (type) {
			case 'number':
				useFirst = selector === testValue;
				break;
				
			case 'string':
				// If a string is all digits, convert to a number and test that
				if (/^\d+$/.test(selector)) {
					useFirst = parseInt(selector, 10) === testValue;
				}
				else {
					useFirst = testValue === 0 ? selector.length === 0 : false;
				}
				break;
				
			default:
				// If we are testing against zero,
				// consider null, undefined, or boolean false as zero
				useFirst = testValue === 0 ? !selector : false;
				break;
		}
		
		return useFirst ? choice1 : choice2;
	}
	
	
	function doFormat(format, args) {
		if (format.length === 0) {
			return '';
		}
				
		for (var i = 0; i <= 1; ++i) {
			// Create a RegExp that will match the selector pattern
			var re = String.format('[{0}](.+?)[{0}](.*?)[{0}](.*?)[{0}]', Ext.escapeRe(Ext.ux.util.Format.formatDelimiters.charAt(i)));
			
			// The replace function needs the test value (0 or 1) and the original arguments
			var delegate = formatReplace.createDelegate(this, [i, args], 4);
			
			// Do a global replace using formatReplace()
			format = format.replace(new RegExp(re, 'g'), delegate);
		}
		
		// Finally we do regular parameter replacement
		format = String.format.apply(String.format, [format].concat(args));
		return format;
	}
		
	
	return {
		formatDelimiters: '#|',
	
	
		xformat: function(format) {
			var args = Array.prototype.slice.call(arguments, 1, arguments.length);
			return doFormat(format, args);
		},
		
		
		xformatRenderer: function(format) {
			return function() {
				var args = Array.prototype.slice.call(arguments, 0, arguments.length);
				return doFormat(format, args);
			};
		}
	};
}();

String.xformat = Ext.ux.util.Format.xformat;

Ext.applyIf(String, {
	
	stripPrefix: function(s, greedy, separator) {
		var sep = separator || '-';
		var r = new RegExp(String.format('^.+{0}{1}(.*)', (greedy ? '' : '?'), sep));
		var m = s.match(r);
		
		if (m) {
			return m[1];
		}
		else {
			return s;
		}
	}
});


Ext.ux.util.ImagePreloader = function(images, progressCallback, completionCallback, scope) {
	this.progressCallback = progressCallback;
	this.completionCallback = completionCallback;
	this.scope = scope || this;
	this.numLoaded = 0;
	this.numProcessed = 0;
	this.images = new Array;

	// record the number of images.
	this.numImages = images.length;

	// for each image, call preload()
	for (var i = 0; i < images.length; ++i) {
		this.preload(images[i]);
	}
};
Ext.ux.util.ImagePreloader.prototype = {
	
	preload: function(src)
	{
		var img = new Image;
		this.images.push(img);

		img.onload = this.onLoad;
		img.onerror = this.onError;
		img.onabort = this.onAbort;

		img.preloader = this;
		img.status = '';
		img.src = src;
	},
	
	
	onComplete: function()
	{
		++this.numProcessed;
		
		if (this.progressCallback) {
			this.progressCallback.call(this.scope, this.numProcessed, this.numImages);
		}
		
		if (this.numProcessed == this.numImages) {
			this.completionCallback.call(this.scope, this.images, this.numLoaded);
		}
	},
	
	
	onLoad: function()
	{
		this.status = 'loaded';
		this.preloader.numLoaded++;
		this.preloader.onComplete();
	},
	
	
	onError: function()
	{
		this.status = 'error';
		this.preloader.onComplete();
	},
	
	
	onAbort: function()
	{
		this.status = 'abort';
		this.preloader.onComplete();
	}
};


Ext.ux.Window = {
	
	clipHeight: function(height) {
		return Math.min(Ext.getBody().getHeight() - 20, height);
	},
	
	screenHeight: function() {
		return Ext.getBody().getHeight() - 20;
	}
};


Ext.ux.SearchField = Ext.extend(Ext.form.TwinTriggerField, {
	
	validationEvent: false,
	validateOnBlur: false,
	trigger1Class: 'x-form-clear-trigger',
	trigger2Class: 'x-form-search-trigger',
	hideTrigger1: true,
	width: 180,
	hasSearch: false,
	paramName: 'query',
	minLength: 2,
	trigger1Reloads: true,


	initComponent : function() {
		if (!this.store.baseParams) {
			this.store.baseParams = {};
		}
		
		Ext.ux.SearchField.superclass.initComponent.call(this);
		this.on('specialkey', function(f, e) {
			if (e.getKey() == e.ENTER) {
				this.onTrigger2Click();
			}
		}, this);
	},
	
	onTrigger1Click : function() {
		if (this.hasSearch) {
			this.store.baseParams[this.paramName] = '';
			
			if (this.trigger1Reloads) {
				this.store.reload({params: {start: 0}});
			}
			else {				
				this.store.removeAll();
			}
			
			this.el.dom.value = '';
			this.triggers[0].hide();
			this.hasSearch = false;
			this.focus();
		}
	},

	onTrigger2Click : function() {
		var v = this.getRawValue();

		if (v.length < 1) {
			this.onTrigger1Click();
			return;
		}

		if (v.length < this.minLength) {
			Ext.Msg.alert('Invalid Search', 'You must enter a minimum of ' + this.minLength + ' characters.');
			return;
		}
		
		this.store.baseParams[this.paramName] = v;
		var o = {start: 0};
		this.store.reload({params:o});
		this.hasSearch = true;
		this.triggers[0].show();
		this.focus();
	}
});
